@@ -12,6 +12,7 @@
 #ifdef __APPLE__
 #include <sys/attr.h>
 #include <sys/paths.h>
+#include <sys/xattr.h>
 #endif /* __APPLE__ */
 #include <errno.h>
 #include <fcntl.h>
@@ -31,6 +32,7 @@
 
 #include <snet.h>
 
+#include "openssl_compat.h" // Compatibility shims for OpenSSL < 1.1.0
 #include "applefile.h"
 #include "connect.h"
 #include "cksum.h"
@@ -74,8 +76,8 @@ retr( SNET *sn, char *pathdesc, char *path, char *temppath, mode_t tempmode,
     char		buf[ 8192 ]; 
     ssize_t		rr;
     extern EVP_MD	*md;
-    EVP_MD_CTX		mdctx;
-    unsigned char	md_value[ EVP_MAX_MD_SIZE ];
+    EVP_MD_CTX		*mdctx = EVP_MD_CTX_new();
+    unsigned char	md_value[ SZ_BASE64_D( SZ_BASE64_E( EVP_MAX_MD_SIZE ) ) ];
     char		cksum_b64[ SZ_BASE64_E( EVP_MAX_MD_SIZE ) ];
 
     if ( cksum ) {
@@ -84,7 +86,7 @@ retr( SNET *sn, char *pathdesc, char *path, char *temppath, mode_t tempmode,
 	    fprintf( stderr, "%s\n", pathdesc );
 	    return( 1 );
 	}
-	EVP_DigestInit( &mdctx, md );
+	EVP_DigestInit( mdctx, md );
     }
 
     if ( verbose ) printf( ">>> RETR %s\n", pathdesc );
@@ -165,7 +167,7 @@ retr( SNET *sn, char *pathdesc, char *path, char *temppath, mode_t tempmode,
 	    goto error2;
 	}
 	if ( cksum ) {
-	    EVP_DigestUpdate( &mdctx, buf, (unsigned int)rr );
+	    EVP_DigestUpdate( mdctx, buf, (unsigned int)rr );
 	}
 	if ( dodots ) { putc( '.', stdout ); fflush( stdout ); }
 	size -= rr;
@@ -197,8 +199,9 @@ retr( SNET *sn, char *pathdesc, char *path, char *temppath, mode_t tempmode,
 
     /* cksum file */
     if ( cksum ) {
-	EVP_DigestFinal( &mdctx, md_value, &md_len );
+	EVP_DigestFinal( mdctx, md_value, &md_len );
 	base64_e( md_value, md_len, cksum_b64 );
+	EVP_MD_CTX_free(mdctx);
 	if ( strcmp( trancksum, cksum_b64 ) != 0 ) {
 	    fprintf( stderr, "line %d: checksum in transcript does not match "
 		"checksum from server\n", linenum );
@@ -246,8 +249,8 @@ retr_applefile( SNET *sn, char *pathdesc, char *path, char *temppath,
     struct as_entry		ae_ents[ 3 ]; 
     struct timeval		tv;
     extern EVP_MD       	*md;
-    EVP_MD_CTX   	       	mdctx;
-    unsigned char       	md_value[ EVP_MAX_MD_SIZE ];
+    EVP_MD_CTX   	       	*mdctx;
+    unsigned char       	md_value[ SZ_BASE64_D( SZ_BASE64_E( EVP_MAX_MD_SIZE ) ) ];
     char		       	cksum_b64[ SZ_BASE64_E( EVP_MAX_MD_SIZE ) ];
 
     if ( cksum ) {
@@ -256,7 +259,7 @@ retr_applefile( SNET *sn, char *pathdesc, char *path, char *temppath,
 	    fprintf( stderr, "%s\n", pathdesc );
             return( 1 );
         }
-        EVP_DigestInit( &mdctx, md );
+        EVP_DigestInit( mdctx, md );
     }
 
     if ( verbose ) printf( ">>> RETR %s\n", pathdesc );
@@ -316,7 +319,7 @@ retr_applefile( SNET *sn, char *pathdesc, char *path, char *temppath,
 	return( -1 );
     }
     if ( cksum ) {
-	EVP_DigestUpdate( &mdctx, (char *)&ah, (unsigned int)rc );
+	EVP_DigestUpdate( mdctx, (char *)&ah, (unsigned int)rc );
     }
 
     /* name temp file */
@@ -373,7 +376,7 @@ retr_applefile( SNET *sn, char *pathdesc, char *path, char *temppath,
     /* Should we check for valid ae_ents here? YES! */
 
     if ( cksum ) {
-	EVP_DigestUpdate( &mdctx, (char *)&ae_ents, (unsigned int)rc );
+	EVP_DigestUpdate( mdctx, (char *)&ae_ents, (unsigned int)rc );
     }
     if ( dodots ) { putc( '.', stdout ); fflush( stdout ); }
 
@@ -398,7 +401,7 @@ retr_applefile( SNET *sn, char *pathdesc, char *path, char *temppath,
 	goto error2;
     }
     if ( cksum ) {
-	EVP_DigestUpdate( &mdctx, finfo, (unsigned int)rc );
+	EVP_DigestUpdate( mdctx, finfo, (unsigned int)rc );
     }
     if ( dodots ) { putc( '.', stdout ); fflush( stdout ); }
     size -= rc;
@@ -425,6 +428,17 @@ retr_applefile( SNET *sn, char *pathdesc, char *path, char *temppath,
 	    goto error2;
 	}
 
+	/* APFS won't create a new resource fork with _PATH_RSRCFORKSPEC - ugh */
+	const char *rsrcvalue = "";
+	int rsrcresult = fsetxattr( dfd, XATTR_RESOURCEFORK_NAME, &rsrcvalue,
+		sizeof(rsrcvalue), 0, 0 );
+	if ( rsrcresult == -1 ) {
+	    fprintf( stderr,"%s: Unable to create resource fork %s\n",rsrc_path, strerror( errno ));
+	    returnval = -1;
+	    goto error2;
+	}
+	
+
 	/* No need to mkprefix as dfd is already present */
 	if (( rfd = open( rsrc_path, O_WRONLY, 0 )) < 0 ) {
 	    perror( rsrc_path );
@@ -448,7 +462,7 @@ retr_applefile( SNET *sn, char *pathdesc, char *path, char *temppath,
 		goto error3;
 	    }
 	    if ( cksum ) {
-		EVP_DigestUpdate( &mdctx, buf, (unsigned int)rc );
+		EVP_DigestUpdate( mdctx, buf, (unsigned int)rc );
 	    }
 	    if ( dodots ) { putc( '.', stdout ); fflush( stdout ); }
 	    if ( showprogress ) {
@@ -482,7 +496,7 @@ retr_applefile( SNET *sn, char *pathdesc, char *path, char *temppath,
 	}
 
 	if ( cksum ) {
-	    EVP_DigestUpdate( &mdctx, buf, (unsigned int)rc );
+	    EVP_DigestUpdate( mdctx, buf, (unsigned int)rc );
 	}
 	if ( dodots ) { putc( '.', stdout ); fflush( stdout); }
 	if ( showprogress ) {
@@ -523,8 +537,9 @@ retr_applefile( SNET *sn, char *pathdesc, char *path, char *temppath,
     if ( verbose ) printf( "<<< .\n" );
 
     if ( cksum ) {
-	EVP_DigestFinal( &mdctx, md_value, &md_len );
+	EVP_DigestFinal( mdctx, md_value, &md_len );
 	base64_e(( char*)&md_value, md_len, cksum_b64 );
+        EVP_MD_CTX_free(mdctx);
         if ( strcmp( trancksum, cksum_b64 ) != 0 ) {
 	    fprintf( stderr, "line %d: checksum in transcript does not match "
 		"checksum from server\n", linenum );
