@@ -23,6 +23,7 @@
 #include <sys/vfs.h>
 #endif /* linux */
 
+#include "openssl_compat.h" // Compatibility shims for OpenSSL < 1.1.0
 #include "applefile.h"
 #include "base64.h"
 #include "transcript.h"
@@ -38,6 +39,7 @@ extern int	showprogress;
 extern off_t	lsize;
 extern char	*version;
 
+int		tran_format = -1;
 int		cksum = 0;
 int		force = 0;
 int		case_sensitive = 1;
@@ -61,9 +63,9 @@ copy_file( struct transcript *t, char *dst, char *src, int where )
     char		*trancksum = t->t_pinfo.pi_cksum_b64;
     char		*path = t->t_pinfo.pi_name;
     ssize_t		rr, size = 0;
-    EVP_MD_CTX          mdctx;
-    unsigned char       md_value[ EVP_MAX_MD_SIZE ];
-    char       		cksum_b64[ SZ_BASE64_E( EVP_MAX_MD_SIZE ) ];
+    EVP_MD_CTX          *mdctx = EVP_MD_CTX_new();
+    unsigned char       md_value[ SZ_BASE64_D( SZ_BASE64_E( EVP_MAX_MD_SIZE ) ) ];
+    char       	        cksum_b64[ SZ_BASE64_E( EVP_MAX_MD_SIZE ) ];
 #ifdef __APPLE__
     extern struct as_header as_header;
     extern struct attrlist  getalist;
@@ -83,7 +85,7 @@ copy_file( struct transcript *t, char *dst, char *src, int where )
 	    fprintf( stderr, "line %d: no checksum\n", t->t_linenum );
 	    return( -1 );
 	}
-	EVP_DigestInit( &mdctx, md );
+	EVP_DigestInit( mdctx, md );
     }
 
     if (( rfd = open( src, O_RDONLY, 0 )) < 0 ) {
@@ -150,7 +152,7 @@ copy_file( struct transcript *t, char *dst, char *src, int where )
 	    goto error2;
 	}
 	if ( cksum ) {
-	    EVP_DigestUpdate( &mdctx, ( char * )&header, ( unsigned int )rr );
+	    EVP_DigestUpdate( mdctx, ( char * )&header, ( unsigned int )rr );
 	}
 	size -= rr;
 	if ( showprogress ) {
@@ -175,7 +177,7 @@ copy_file( struct transcript *t, char *dst, char *src, int where )
 	as_entry_netswap( &as_ents[ AS_DFE ] );
 
 	if ( cksum ) {
-	    EVP_DigestUpdate( &mdctx, ( char * )&as_ents, ( unsigned int )rr );
+	    EVP_DigestUpdate( mdctx, ( char * )&as_ents, ( unsigned int )rr );
 	}
 	size -= rr;
 	if ( showprogress ) {
@@ -198,7 +200,7 @@ copy_file( struct transcript *t, char *dst, char *src, int where )
 	    goto error2;
 	}
 	if ( cksum ) {
-	    EVP_DigestUpdate( &mdctx, finfo, ( unsigned int )rr );
+	    EVP_DigestUpdate( mdctx, finfo, ( unsigned int )rr );
 	}
 	if ( showprogress ) {
 	    progressupdate( rr, path );
@@ -229,7 +231,7 @@ copy_file( struct transcript *t, char *dst, char *src, int where )
 		goto error2;
 	    }
 	    if ( cksum ) {
-		EVP_DigestUpdate( &mdctx, buf, ( unsigned int )rr );
+		EVP_DigestUpdate( mdctx, buf, ( unsigned int )rr );
 	    }
 	    if ( showprogress ) {
 		progressupdate( rr, path );
@@ -248,7 +250,7 @@ copy_file( struct transcript *t, char *dst, char *src, int where )
 	 * entries to the checksum digest.
 	 */
 	if ( cksum ) {
-	    EVP_DigestUpdate( &mdctx, &as_header, AS_HEADERLEN );
+	    EVP_DigestUpdate( mdctx, &as_header, AS_HEADERLEN );
 
 	    as_ents[AS_FIE].ae_id = ASEID_FINFO;
 	    as_ents[AS_FIE].ae_offset = AS_HEADERLEN +
@@ -274,7 +276,7 @@ copy_file( struct transcript *t, char *dst, char *src, int where )
 	    as_entry_netswap( &as_ents[ AS_RFE ] );
 	    as_entry_netswap( &as_ents[ AS_DFE ] );
 
-	    EVP_DigestUpdate( &mdctx, ( char * )&as_ents,
+	    EVP_DigestUpdate( mdctx, ( char * )&as_ents,
 					    ( 3 * sizeof( struct as_entry )));
 	} 
 	size -= ( AS_HEADERLEN + ( 3 * sizeof( struct as_entry )) + FINFOLEN );
@@ -292,7 +294,7 @@ copy_file( struct transcript *t, char *dst, char *src, int where )
 	    }
 	}
 	if ( cksum ) {
-	    EVP_DigestUpdate( &mdctx, ai.ai_data, FINFOLEN );
+	    EVP_DigestUpdate( mdctx, ai.ai_data, FINFOLEN );
 	}
 
 	/* read and write the finder info and rsrc fork from the system */
@@ -318,7 +320,7 @@ copy_file( struct transcript *t, char *dst, char *src, int where )
 		    goto error2;
 		}
 		if ( cksum ) {
-		    EVP_DigestUpdate( &mdctx, buf, rr );
+		    EVP_DigestUpdate( mdctx, buf, rr );
 		}
 		size -= rr;
 		if ( showprogress ) {
@@ -354,7 +356,7 @@ copy_file( struct transcript *t, char *dst, char *src, int where )
 	    goto error2;
 	}
 	if ( cksum ) {
-	    EVP_DigestUpdate( &mdctx, buf, rr );
+	    EVP_DigestUpdate( mdctx, buf, rr );
 	}
 	size -= rr;
 	if ( showprogress ) {
@@ -383,8 +385,9 @@ copy_file( struct transcript *t, char *dst, char *src, int where )
     }
 
     if ( cksum ) {
-	EVP_DigestFinal( &mdctx, md_value, &md_len );
+	EVP_DigestFinal( mdctx, md_value, &md_len );
 	base64_e( md_value, md_len, ( char * )cksum_b64 );
+        EVP_MD_CTX_free(mdctx);
 	if ( strcmp( trancksum, cksum_b64 ) != 0 ) {
 	    if ( force ) {
 		fprintf( stderr, "warning: " );
@@ -556,7 +559,7 @@ main( int ac, char *av[] )
     struct stat		st;
     char		*transcript;
     char		shortname[ MAXPATHLEN ], fullpath[ MAXPATHLEN ];
-    char		tmp[ MAXPATHLEN ], src[ MAXPATHLEN ], dst[ MAXPATHLEN ];
+    char		src[ MAXPATHLEN ], dst[ MAXPATHLEN ];
     char		root[ MAXPATHLEN ] = "/";
     char		*kfile = "/dev/null";
     char		*p, *dstdir = NULL;
@@ -718,31 +721,23 @@ main( int ac, char *av[] )
 	}
 
 	/* create the destination path */
-	if ( snprintf( tmp, MAXPATHLEN, "%s/%s", dstdir, t->t_pinfo.pi_name )
+	if ( snprintf( dst, MAXPATHLEN, "%s/%s", dstdir, t->t_pinfo.pi_name )
 		>= MAXPATHLEN ) {
 	    fprintf( stderr, "%s/%s: path too long\n", dstdir,
 			t->t_pinfo.pi_name );
 	    exit( 2 );
 	}
-	if ( mkdirs( tmp ) < 0 ) {
-	    fprintf( stderr, "mkdirs %s: %s\n", tmp, strerror( errno ));
-	}
-	if ( realpath( tmp, dst ) == NULL ) {
-	    fprintf( stderr, "realpath %s: %s\n", tmp, strerror( errno ));
-	    exit( 2 );
+	if ( mkdirs( dst ) < 0 ) {
+	    fprintf( stderr, "mkdirs %s: %s\n", dst, strerror( errno ));
 	}
 
 	/* and the source path */
-	if ( snprintf( tmp, MAXPATHLEN, "%s/%s", root, t->t_pinfo.pi_name )
+	if ( snprintf( src, MAXPATHLEN, "%s/%s", root, t->t_pinfo.pi_name )
 		>= MAXPATHLEN ) {
 	    fprintf( stderr, "%s/%s: path too long\n", dstdir,
 			t->t_pinfo.pi_name );
 	    exit( 2 );
 	}
-	if ( realpath( tmp, src ) == NULL ) {
-	    fprintf( stderr, "realpath %s: %s\n", tmp, strerror( errno ));
-	    exit( 2 );
-	}
 
 	if ( local_update( t, dst, src, where ) != 0 ) {
 	    /* XXX is this really a good idea? */
