@@ -3,6 +3,6 @@
  * All Rights Reserved.  See COPYRIGHT.
  */
 
-int tls_server_setup( int use_randfile, int authlevel, char *caFile, char *caDir, char *cert, char *privatekey );
+int tls_server_setup( int use_randfile, int authlevel, char *caFile, char *caDir, char *crlFile, char *crlDir, char *cert, char *privatekey );
 int tls_client_setup( int use_randfile, int authlevel, char *caFile, char *caDir, char *cert, char *privatekey );
 int tls_client_start( SNET *sn, char *host, int authlevel );
